#!/usr/bin/perl
#
#    reports.pl - NMIS Perl Program - Network Mangement Information System
#    Copyright (C) 2000 Sinclair InterNetworking Services Pty Ltd
#    Keith Sinclair <keith@sinclair.org.au>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#*****************************************************************************
# Auto configure to the <nmis-base>/lib and <nmis-base>/files/nmis.conf
use lib "/usr/share/nmis/lib";
# 
#****** Shouldn't be anything else to customise below here *******************

# See if valid options exist!!!
if ( $#ARGV <= 0 ) {
	print "$0 generates reports from the STATS system\n";
	print "command line options are:\n";
	print "\t[type=health] - Type of report\n";
	print "\t[length=day|week|month] - Length of report\n";
	print "\t[outfile=filename] - Output file for web report, don't use for STDOUT\n";
	print "\t[file=filename] - STATS Configuration File\n";
	print "\n";
	
	exit(1);
}

use RRDs; 
use strict;
use web;
use NMIS;
use func;
								       
my %nvp = getArguements(@ARGV);

# Set debugging level.
my $debug = setDebug($nvp{debug});
$NMIS::debug = $debug;

#Global Time Stamp from when set date is.
my ($head_string, $start_string, $start, $end);
my ($database, $adminCellColor, $operCellColor, $availCellColor, $operAvailability, $totalUtil, $totalCellColor, $intString);

# See if customised config file required.
my $conf;
if ( $nvp{file} ne "" ) { $conf = $nvp{file}; }
else { $conf = "nmis.conf"; }
my $configfile = "/etc/nmis/$conf";
if ( -f $configfile ) { loadConfiguration($configfile); }
else { die "Can't access configuration file $configfile.\n"; }


loadConfiguration($configfile);

# make the default report type = health
if ( $nvp{type} eq "" ) { $nvp{type} = "health"; }

# convert length to something ie 1 month is 1 oct to 30 oct!!!!

&setReportLength;

&typeReports; 


sub typeReports {
	my $net;
	my $role;
	my $time = time;
	my $date = returnDateStamp;
	my $graphLink;
	my $header;
	my $reportLength;
	my $random = rand 10000;
	my $reportnode;
	my $index;
	my $reportStats;
	my $cellColor;
	my %reportTable;
	my %summaryTable;
	my @tmparray;
	my @tmpsplit;
	my @logreport;
	my @logline;
	my @spacesplit;
	my @summaryOrder = ("lan-core","lan-distribution","lan-access","wan-core","wan-distribution","wan-access");
	my $logfile = $NMIS::config{event_log};
	my $outageLength;
	my $outageColor;
	my $outageDetails;
	my $outageType;
	my $summaryhash;

	NMIS::loadNodeDetails;

	open (STDOUT,">$nvp{outfile}") or die "Cannot open the file $nvp{outfile}. $!";
	
	if ( $nvp{type} eq "health" ) {

		pageStartFile("Summary Health Metrics",$NMIS::config{styles});
	 	tableStart("#FFFFFF","");
	
		# Get each of the nodes info in a HASH for playing with
		foreach $reportnode (sort(keys(%NMIS::nodeTable))) {
			NMIS::loadSystemFile($reportnode);

			$reportStats = NMIS::summaryStats($reportnode,"health",$start,$end);
			if ( $reportStats != 1 ) {
				@tmparray = split " ","@$reportStats";
				for ( $index = 0; $index <= $#tmparray; ++$index ) {
					@tmpsplit = split "=",@tmparray[$index];
					$reportTable{$reportnode}{$tmpsplit[0]} = $tmpsplit[1] ;
				}
			}
			$reportTable{$reportnode}{net} = $NMIS::nodeTable{$reportnode}{net};
			$reportTable{$reportnode}{role} = $NMIS::nodeTable{$reportnode}{role};
			$reportTable{$reportnode}{devicetype} = $NMIS::nodeTable{$reportnode}{devicetype};

			# Calculate the summaries!!!!
			$summaryhash = "$NMIS::nodeTable{$reportnode}{net}-$NMIS::nodeTable{$reportnode}{role}";
			$summaryTable{$summaryhash}{net} = $NMIS::nodeTable{$reportnode}{net};
			$summaryTable{$summaryhash}{role} = $NMIS::nodeTable{$reportnode}{role};
			$summaryTable{$summaryhash}{reachable} = $summaryTable{$summaryhash}{reachable} + $reportTable{$reportnode}{reachable};
			$summaryTable{$summaryhash}{available} = $summaryTable{$summaryhash}{available} + $reportTable{$reportnode}{available};
			$summaryTable{$summaryhash}{health} = $summaryTable{$summaryhash}{health} + $reportTable{$reportnode}{health};
			$summaryTable{$summaryhash}{response} = $summaryTable{$summaryhash}{response} + $reportTable{$reportnode}{response};
			++$summaryTable{$summaryhash}{count};
		}

		printRow("<FONT SIZE=+2><STRONG>$head_string</STRONG></FONT>","#FFFFFF",1);

	 	rowStart;
	 	cellStart("#FFFFFF",20);

	 	tableStart("white");

	 	printHeadRow("Average Health for Device Groups","#FFFFFF",10);							
	 	printHeadRow("Net,Role,Reachability,Interface Avail.,Health,Response Time","#FFFFFF");
															
		# print a summary table for the health
		for ($index = 0; $index <= $#summaryOrder; ++$index) {
			rowStart;
			printHeadCell("<A HREF=\"#$summaryOrder[$index]\">".uc($summaryTable{$summaryOrder[$index]}{net})."</A>");
			printHeadCell("<A HREF=\"#$summaryOrder[$index]\">".$summaryTable{$summaryOrder[$index]}{role}."</A>");
			if ( $summaryTable{$summaryOrder[$index]}{reachable} > 0 ) {
				$summaryTable{$summaryOrder[$index]}{avgreachable} = sprintf("%.3f",$summaryTable{$summaryOrder[$index]}{reachable} / $summaryTable{$summaryOrder[$index]}{count}); 
				$cellColor = NMIS::colorHighGood($summaryTable{$summaryOrder[$index]}{avgreachable});
				printHeadCell($summaryTable{$summaryOrder[$index]}{avgreachable},$cellColor);
			}
			if ( $summaryTable{$summaryOrder[$index]}{available} > 0 ) { 
				$summaryTable{$summaryOrder[$index]}{avgavailable} = sprintf("%.3f",$summaryTable{$summaryOrder[$index]}{available} / $summaryTable{$summaryOrder[$index]}{count}); 
				$cellColor = NMIS::colorHighGood($summaryTable{$summaryOrder[$index]}{avgavailable});
				printHeadCell($summaryTable{$summaryOrder[$index]}{avgavailable},$cellColor);
			}
			if ( $summaryTable{$summaryOrder[$index]}{health} > 0 ) { 
				$summaryTable{$summaryOrder[$index]}{avghealth} = sprintf("%.3f",$summaryTable{$summaryOrder[$index]}{health} / $summaryTable{$summaryOrder[$index]}{count}); 
				$cellColor = NMIS::colorHighGood($summaryTable{$summaryOrder[$index]}{avghealth});
				printHeadCell($summaryTable{$summaryOrder[$index]}{avghealth},$cellColor);
			}
			if ( $summaryTable{$summaryOrder[$index]}{response} > 0 ) { 
				$summaryTable{$summaryOrder[$index]}{avgresponse} = sprintf("%.3f",$summaryTable{$summaryOrder[$index]}{response} / $summaryTable{$summaryOrder[$index]}{count}); 
				$cellColor = NMIS::colorResponseTime($summaryTable{$summaryOrder[$index]}{avgresponse});
				printHeadCell($summaryTable{$summaryOrder[$index]}{avgresponse},$cellColor);
			}
			rowEnd;
		}

	 	tableEnd;
		paragraph;
	 	tableStart("white");
		 	printHeadRow("<A NAME=lan-core><A HREF=#TOP>LAN Core</A>","#FFFFFF",10);
		 	printHeadRow("Node,Device Type,Reachability,Interface Avail.,Health,Response Time","#FFFFFF");
			&reportNodeType("lan","core");
	 	tableEnd;
		paragraph;
	 	tableStart("white");
		 	printHeadRow("<A NAME=lan-distribution><A HREF=#TOP>LAN Distribution</A>","#FFFFFF",10);
		 	printHeadRow("Node,Device Type,Reachability,Interface Avail.,Health,Response Time","#FFFFFF");
			&reportNodeType("lan","distribution");
	 	tableEnd;
		paragraph;
	 	tableStart("white");
		 	printHeadRow("<A NAME=lan-access><A HREF=#TOP>LAN Access</A>","#FFFFFF",10);
		 	printHeadRow("Node,Device Type,Reachability,Interface Avail.,Health,Response Time","#FFFFFF");
			&reportNodeType("lan","access");
	 	tableEnd;
		paragraph;
	 	tableStart("white");
		 	printHeadRow("<A NAME=wan-core><A HREF=#TOP>WAN Core</A>","#FFFFFF",10);
		 	printHeadRow("Node,Device Type,Reachability,Interface Avail.,Health,Response Time","#FFFFFF");
			&reportNodeType("wan","core");
	 	tableEnd;
		paragraph;
	 	tableStart("white");
		 	printHeadRow("<A NAME=wan-distribution><A HREF=#TOP>WAN Distribution</A>","#FFFFFF",10);
		 	printHeadRow("Node,Device Type,Reachability,Interface Avail.,Health,Response Time","#FFFFFF");
			&reportNodeType("wan","distribution");
	 	tableEnd;
		paragraph;
	 	tableStart("white");
		 	printHeadRow("<A NAME=wan-access><A HREF=#TOP>WAN Access</A>","#FFFFFF",10);
		 	printHeadRow("Node,Device Type,Reachability,Interface Avail.,Health,Response Time","#FFFFFF");
			&reportNodeType("wan","access");
	 	tableEnd;

	 	cellEnd;
	 	rowEnd;
	} # reporttype eq health
	elsif ( $nvp{type} eq "outage" ) {
		my $outage;
	 	# Open the file and store in table
		open (DATA, $logfile) or die "Cannot open the file $logfile. $!";

		# find the line with the entry in and store in the array
		while (<DATA>) {
			chomp;
			@logreport[$index] = $_;		      
			++$index;
		}

		pageStartFile("Outage Report",$NMIS::config{styles});
	 	tableStart("#FFFFFF","");
				   
	    	headerBar("<STRONG>Outage Report @ $date</STRONG>",1);
	
	 	rowStart;
	 	cellStart("#FFFFFF",20);

	 	tableStart("white");

	       		printHeadRow("Time,Node,Outage Type,Outage Time,Interface,Planned Outage","#FFFFFF");
			for ($index = 0; $index <= $#logreport; ++$index){
			 	if ( $logreport[$index] =~ /up/i ) {
					@logline = split ( "," , $logreport[$index]);
				  	if ($logline[2] =~ /node/i ){ 
						$outageType = "Node Outage";
						$outageLength = $logline[4];
						$outageLength =~ s/Time=(.*) secs/$1/; 
						$outageLength = NMIS::convertSecsHours($outageLength);
						$outageColor = NMIS::colorTime($outageLength);
						$outageDetails = "";
						$outage =  $logline[4];
						$outage =~ s/interface=.* secs|Time=.*secs |Time=.*secs//gi; 
						$outage =~ s/tics=(.*)/$1/i; 
						if ( $outage ne "" ) {
							$outage = "TICS=$outage";
						}
					}# End of if node.
					else {
						@spacesplit = split ( " " , $logline[4]);
						$spacesplit[1] =~ s/Time=(.*)/$1/; 
						$spacesplit[0] =~ s/Interface=//i; 
						$outageType = "Interface Outage";
						$outageLength = NMIS::convertSecsHours($spacesplit[1]);
						$outageColor = NMIS::colorTime($outageLength);
						$outageDetails = $spacesplit[0];
					}

					if ( $logreport[$index] =~ /Time=/ ) {
						rowStart;
						printCell(returnDateStamp($logline[0]),$outageColor);
						printCell($logline[1],$outageColor);
						printCell($outageType,$outageColor);
						printCell($outageLength,$outageColor);
						printCell($outageDetails,$outageColor);
						printCell($outage,$outageColor);
						rowEnd;
					}
				} #End if up.
			} # End for loop.

		tableEnd;															
	 	cellEnd;
	 	rowEnd;
	} # end of report = outage
	elsif ( $nvp{type} eq "response" ) {
		$reportLength = "-8 hours";
		$header = "Reponse Time Summary - 8 Hour Snapshot @ $date"; 

		pageStartFile("Summary Response Time",$NMIS::config{styles});
	 	tableStart("#FFFFFF","");

	    	headerBar("$header",1);
	
		# Get each of the nodes info in a HASH for playing with
	       	foreach $reportnode (sort(keys(%NMIS::nodeTable))) {

			NMIS::loadSystemFile($reportnode);

    			$reportStats = NMIS::summaryStats($reportnode,"health",$reportLength,time);
			if ( $reportStats != 1 ) {
				@tmparray = split " ","@$reportStats";
				for ( $index = 0; $index <= $#tmparray; ++$index ) {
					@tmpsplit = split "=",@tmparray[$index];
					$reportTable{$reportnode}{$tmpsplit[0]} = $tmpsplit[1] ;
				}
			}
			$reportTable{$reportnode}{net} = $NMIS::nodeTable{$reportnode}{net};
			$reportTable{$reportnode}{role} = $NMIS::nodeTable{$reportnode}{role};
			$reportTable{$reportnode}{devicetype} = $NMIS::nodeTable{$reportnode}{devicetype};
		}

	 	rowStart;
	 	cellStart("#FFFFFF",20);

	 	tableStart("white");
	 	printHeadRow("Average Response Time for Routers","#FFFFFF",10);							
		 	rowStart;
			printHeadCell("Node","#FFFFFF");
		 	printHeadCell("Node Type","#FFFFFF");
		 	printHeadCell("Response Time","#FFFFFF");
		 	rowEnd;
			foreach $reportnode ( sort { $reportTable{$b}{response} cmp $reportTable{$a}{response} } keys %reportTable ) 
			{
				NMIS::loadSystemFile($reportnode);
				if ( $reportTable{$reportnode}{devicetype} eq "router" ) {
					rowStart;
					printCell("<A HREF=\"$NMIS::config{nmis}?type=info&node=$reportnode\">$reportnode</A>");
					printCell("$reportTable{$reportnode}{devicetype}");
					$cellColor = NMIS::colorResponseTime($reportTable{$reportnode}{response});
					printCellRight("$reportTable{$reportnode}{response}",$cellColor);
					rowEnd;
				}
			}
	 	tableEnd;

	 	cellEnd;
	 	rowEnd;
	} # report eq response


	tableEnd;
	pageEnd;
	newLine;

	sub reportNodeType {				  

		my $netType = shift;
		my $roleType = shift;
		foreach $reportnode ( sort 
			{ 
				$reportTable{$a}{reachable} cmp $reportTable{$b}{reachable} ||
				$reportTable{$a}{available} cmp $reportTable{$b}{available} 
			} 
			keys %reportTable ) 
		{
			NMIS::loadSystemFile($reportnode);
			if (	( $reportTable{$reportnode}{net} eq "$netType" )  &&
				( $reportTable{$reportnode}{role} eq "$roleType" ) 
			) {
				rowStart;
				printCell("<A HREF=\"$NMIS::config{nmis}?type=info&node=$reportnode\">$reportnode</A>");
				printCell("$reportTable{$reportnode}{devicetype}");
				$cellColor = NMIS::colorHighGood($reportTable{$reportnode}{reachable});
				printCellRight("$reportTable{$reportnode}{reachable}",$cellColor);
				$cellColor = NMIS::colorHighGood($reportTable{$reportnode}{available});
				printCellRight("$reportTable{$reportnode}{available}",$cellColor);
				$cellColor = NMIS::colorHighGood($reportTable{$reportnode}{health});
				printCellRight("$reportTable{$reportnode}{health}",$cellColor);
				$cellColor = NMIS::colorResponseTime($reportTable{$reportnode}{response});
				printCellRight("$reportTable{$reportnode}{response}",$cellColor);
				rowEnd;
			}
		}
		# Print the summaries!!!!
		$summaryhash = "$netType-$roleType";
		rowStart;
		printHeadCell("Group Averages","#FFFFFF",2);
			$cellColor = NMIS::colorHighGood($summaryTable{$summaryhash}{avgreachable});
			printHeadCell($summaryTable{$summaryhash}{avgreachable},$cellColor);
			$cellColor = NMIS::colorHighGood($summaryTable{$summaryhash}{avgavailable});
			printHeadCell($summaryTable{$summaryhash}{avgavailable},$cellColor);
			$cellColor = NMIS::colorHighGood($summaryTable{$summaryhash}{avghealth});
			printHeadCell($summaryTable{$summaryhash}{avghealth},$cellColor);
			$cellColor = NMIS::colorResponseTime($summaryTable{$summaryhash}{avgresponse});
			printHeadCell($summaryTable{$summaryhash}{avgresponse},$cellColor);
		rowEnd;

	}

	close (STDOUT);

} # end typeReports

sub setReportLength {

	my $datestamp_start;
	my $datestamp_end;

	$datestamp_end = returnDateStamp(time);

	if ( $nvp{length} =~ /day|daily/ ) {
		$start = "-1 day";
		$end = time;
		$start_string = NMIS::convertTime("1day");
		$datestamp_start = returnDateStamp($start_string);
		$head_string = "Daily Summary Health Metrics $datestamp_start to $datestamp_end";
	}
	elsif ( $nvp{length} =~ /week/ ) {
		$start = "-1 week";
		$end = time;
		$start_string = NMIS::convertTime("1week");
		$datestamp_start = returnDateStamp($start_string);
		$head_string = "Weekly Summary Health Metrics $datestamp_start to $datestamp_end";
	}
	# Default is month
	else {	
		$start = "-1 month";
		$end = time;
		$start_string = NMIS::convertTime("1month");
		$datestamp_start = returnDateStamp($start_string);
		$head_string = "Monthly Summary Health Metrics $datestamp_start to $datestamp_end";
	}

}

exit(0);

