#
#    ping.pm - NMIS ping Perl Package - Network Mangement Information System
#    Copyright (C) 2000,2001 Sinclair InterNetworking Services Pty Ltd
#    Keith Sinclair <keith@sinclair.org.au>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
package ping;

require 5;

use strict;

use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

use Exporter;

$VERSION = 1.00;

@ISA = qw(Exporter);

@EXPORT = qw( 
		ping
		pingOld
		responseTime 
		responseTimeOld
	);

@EXPORT_OK = qw( );

use NMIS;
use func;
use Time::HiRes;
use Net::Ping;

sub ping {
	# New ping module
        my %args = @_;
        my $node = $args{node};
        my $timeout = $args{timeout};
        my $debug = $args{debug};
        my $numPings = $args{count};
        my $packetsize = $args{packetsize};
	my $result;
	my $ping;

	if ( $node eq "" ) { 
		print returnTime." NO NODE AS ARGUEMENT\n";
		return 0;			
	}

        if ( $timeout eq "" ) { $timeout = 5; }
        if ( $numPings eq "" ) { $numPings = 5; }
	if ( $packetsize eq "" ) { $packetsize = 64; }
 
	if ( $debug eq "verbose" ) { print returnTime." Pinging $node timeout $timeout\n"; }
	$ping = Net::Ping->new("icmp", $timeout, $packetsize);
	if ( $ping->ping($node, $timeout) ) { $result = 100; } 
	else { $result = 0; } 
	$ping->close;

	if ( $debug eq "verbose" ) { 
		if ( $result == 100 ) { print returnTime." $node is PINGABLE result = $result\n"; } 
		else { print returnTime." $node is NOT REACHABLE result = $result\n"; }
	}

	return $result;
}

sub pingOld {
	# Old Ping for backwards compatibility
        my %args = @_;
        my $node = $args{node};
        my $timeout = $args{timeout};
        my $debug = $args{debug};
        my $numPings = $args{count};

	if ( $debug eq "verbose" ) { print returnTime." Pinging $node timeout $timeout.....\n"; }

	my $ping = "/usr/sbin/ping";
	my $result = `$ping $node $timeout 2>/dev/null`;

	if ( $result =~ /alive/i ) { $result = 100 } 
	else { $result = 0 } 

	return $result;
}

sub responseTime {
	#Written by Richard Kuehnle
	
	# responseTime returns the responseTime of the ping node
        my %args = @_;
        my $node = $args{node};
        my $numPings = $args{count};
        my $timeout = $args{timeout};
        my $packetsize = $args{packetsize};
        my $debug = $args{debug};
	my $i;
        
	if ( $debug eq "verbose" ) { print returnTime." Running Response Time for \"$node\"\n"; }

        if ( $numPings eq "" ) { $numPings = 5; }
        if ( $timeout eq "" ) { $timeout = 5; }
	if ( $packetsize eq "" ) { $packetsize = 64; }

	my $ping = Net::Ping->new("icmp", $timeout, $packetsize);
	my $startTime = Time::HiRes::time();
	for ( $i = 1; $i <= $numPings; $i++ ) { 
		$ping->ping($node, $timeout); 
		if ( $debug eq "verbose" ) { print returnTime." $node ping number $i time=".Time::HiRes::tv_interval([$startTime])."\n"}
	}
	my $pingReturn = sprintf( "%.2f", ((Time::HiRes::tv_interval([$startTime]) / $numPings) *1000));
	if ( $debug eq "true" or $debug eq "verbose" ) { print returnTime." Finished Running Response Time: $node respone time = $pingReturn ms\n"; }
	$ping->close;
	return $pingReturn;
} # responseTime

sub responseTimeOld {
	# This has been included for backwards compatibility
        if ( $NMIS::debug eq "verbose" ) { print returnTime." Running Response Time\n"; }
        my %args = @_;
        my $pingNode = $args{node};
        my $debug = $args{debug};
        my $numPings = $args{count};

        if ( $numPings eq "" ) { $numPings = 5; }

        my $pingOutput = `/usr/sbin/ping -s -n $pingNode 64 $numPings`;
        my @pingOutputArray = split "\/", $pingOutput; 
        if ( $debug eq "true" or $debug eq "verbose" ) { print returnTime." Finished Running Response Time\n"; }
        return $pingOutputArray[3];
} # responseTime

1;
