#
#    web.pm - NMIS Perl Package - Network Mangement Information System
#    Copyright (C) 2000,2001 Sinclair InterNetworking Services Pty Ltd
#    Keith Sinclair <keith@sinclair.org.au>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
package web;

require 5;

use strict;

use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

use Exporter;

$VERSION = 1.00;

@ISA = qw(Exporter);

@EXPORT = qw(
		pageStart
		pageStartCSS
		pageStartFile
		pageEnd
		headerBar
		paragraph
		preStart
		preEnd
		tableStart
		tableEnd
		rowStart
		rowEnd
		cellStart
		cellEnd
		printCell
		printHeadCell
		printCellRight
		printRow
		printRowStats
		printHeadRow
		newLine
		printCGIENV
		errorMessage
		cssPrintCell 
		cssTableStart 
		cssCellStart
		cssHeaderBar
		cssPrintHeadRow
		cssPrintRow
		pageStartView
	);

use NMIS;
use func;

sub printIndent {
	my $i;
	for ($i = 1; $i <= $refer::indent; ++$i) {
		print "\t";
	}
}

sub pageStart {
  	my $string = shift;
  	my $refresh = shift;
	my $refreshstring;
	my $dateStamp = NMIS::returnDateStamp;
	my @dateTime = split " ", $dateStamp;
	$dateTime[0] =~ s/-/ /g;
	if ( $refresh eq "" or $refresh eq "true" ) { $refreshstring = "<META HTTP-EQUIV=Refresh CONTENT=300>"; }
	else { $refreshstring = ""; }
	print <<ENDHTML;
Content-type: text/html\n
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<HEAD>
<link href="$NMIS::config{styles}" rel="stylesheet" type="text/css">
<title>$string</title>
$refreshstring
<META HTTP-EQUIV=expires CONTENT="$dateTime[0]">
</head>

<script language="JavaScript">

	function viewdoc(url,width,height)
	{
		var attrib = 'scrollbars,resizeable,width=' + width + ',height=' + height;
		ViewWindow = window.open(url,"ViewWindow",attrib)
		ViewWindow.focus()
	}
	
</script>
<body>
ENDHTML
}

sub pageStartCSS {
        my $string = shift;
        my $style = shift;
        my $target = shift;
        my $dateStamp = returnDateStamp;
        my @dateTime = split " ", $dateStamp;
        $dateTime[0] =~ s/-/ /g;
        if ($target ne "") { $target = "<base target=\"$target\">"; }
        print <<EOHTML;
Content-type: text/html\n
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<HEAD>
<title>$string</title>
<META HTTP-EQUIV=expires CONTENT="$dateTime[0]">
<link href="$style" rel="stylesheet" type="text/css">
$target
</head>
<body>
EOHTML
}         

sub pageStartView {
  	my $string = shift;
  	my $style = shift;
	my $dateStamp = &returnDateStamp;
	my @dateTime = split " ", $dateStamp;
	$dateTime[0] =~ s/-/ /g;
	print <<EOHTML;
Content-type: text/html\n
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd"> 
<html>
<HEAD>
<title>$string</title>
<META HTTP-EQUIV=expires CONTENT="$dateTime[0]">
<LINK href="$style" REL="stylesheet" TYPE="text/css">
</head>
<BODY LINK="#FFFFFF" VLINK="#FFFFFF">
EOHTML
}

sub pageStartFile {
  	my $string = shift;
  	my $style = shift;

	print <<ENDHTML;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HEAD>
<link href="$style" rel="stylesheet" type="text/css">
<title>$string</title>
</head>

<BODY bgcolor="#FFFFFF" text="#000000" link="#0000FF" vlink="#0000FF">
ENDHTML
}

sub pageEnd { print "</BODY>\n</html>"; }

sub headerBar {
  	my $string = shift;
 	my $span = shift;
	my $spanString = "colspan=6";
	if ( $span ne "" ) { $spanString="colspan=$span "; }
  	print<<ENDHTML;
 <tr>
   <td $spanString bgcolor="#AAAAAA"><BIG><FONT face="Arial" color="#000000">$string</FONT></BIG></td>
 </tr>
ENDHTML
}

sub cssHeaderBar {
  	my $string = shift;
  	my $class = shift;
 	my $span = shift;
	&printIndent;
  	print<<EOHTML;
 <tr>
   <td class="$class" colspan="$span">$string</td>
 </tr>
EOHTML
}

sub cssPrintHeadRow {
  	my $string = shift;
 	my $style = shift;
 	my $span = shift;
	$string =~ s/\n|\t//g ;
	$string =~ s/,/<\/STRONG><\/BIG><\/TD><td class=$style><BIG><STRONG>/g ;
	&printIndent;
	print "<tr><td class=$style colspan=$span><BIG><STRONG>$string</STRONG></BIG></td></tr>";
	print "\n";
}

sub paragraph { print "<P>\n"; }

sub preStart { print "\t<PRE>\n"; }

sub preEnd { print "\t</PRE>\n"; }

sub tableStart {
  	my $string = shift;

	if ($string eq "white") {
	  	print "<table width=100% BORDER=1 bgcolor=#FFFFFF BORDERCOLOR=#000000 BORDERCOLORLIGHT=#000000 BORDERCOLORDARK=#AAAAAA>\n";
	}
	elsif ($string ne "" ) {
	  	print "<table BORDER=0 bgcolor=\"$string\" BORDERCOLOR=#000000 BORDERCOLORLIGHT=#000000 BORDERCOLORDARK=#AAAAAA>\n";
	}
	else {
	  	print "<table BORDER=0 cellpadding=5 cellspacing=5 bgcolor=#190032 bordercolor=#190032 bordercolorlight=#190032 bordercolordark=#190032>\n";
	}
}

sub tableEnd {
  print "</table>\n";
}


sub rowStart {
  print "<tr>\n";
}

sub rowEnd {
  print "</tr>\n";
}

sub cellStart {
  	my $color = shift;
 	my $span = shift;
	if ( $span eq "" ) { $span="1"; }
	if ($color eq "") { $color = "#FFFFFF"; }
	
	print "\t<td width=100% bgcolor=$color colspan=$span>\n"; 
}

sub cellEnd {
  print "\t</td>\n";
}

sub printCell {
  	my $string = shift;
 	my $color = shift;
 	my $colspan = shift;
 	my $align = shift;
	if ( $color eq "" ) { $color="#FFFFFF"; }
	if ( $colspan ne "" ) { $colspan="colspan=\"$colspan\""; }
	if ( $align ne "" ) { $align="ALIGN=\"$align\""; }
	print "\t<td bgcolor=\"$color\" $colspan $align>$string</td>\n";
}

sub printHeadCell {
  	my $string = shift;
 	my $color = shift;
 	my $span = shift;
	my $spanString = "";
	if ( $span ne "" ) { $spanString="colspan=\"$span\" "; }
	if ( $color eq "" ) { $color="#FFFFFF"; }
	print "\t<th bgcolor=\"$color\" $spanString>$string</th>\n";
}

sub printCellRight {
  	my $string = shift;
 	my $color = shift;
 	my $span = shift;
	my $spanString = "";
	if ( $span ne "" ) { $spanString="colspan=$span "; }
	if ( $color eq "" ) { $color="#FFFFFF"; }
	print "\t<td bgcolor=\"$color\" ALIGN=right $spanString>$string</td>\n";
}

sub printRow {
  	my $string = shift;
 	my $color = shift;
 	my $span = shift;
	my $spanString = "";
	if ( $color eq "" ) { $color="#FFFFFF"; }
	if ( $span ne "" ) { $spanString="colspan=$span "; }
	$string =~ s/\n|\t//g ;
	$string =~ s/,/<\/TD><td bgcolor=$color>/g ;
	print "<tr><td bgcolor=\"$color\" $spanString>$string</td></tr>\n";
}

sub printRowStats {
  	my $string = shift;
 	my $color = shift;
 	my $span = shift;
	my $spanString = "";
	if ( $color eq "" ) { $color="#FFFFFF"; }
	if ( $span ne "" ) { $spanString="colspan=$span "; }
	$string =~ s/\n|\t//g ;
	#$string =~ s/ /<\/TD><td bgcolor=$color>/g;
	$string =~ s/\b([a-zA-Z]+)=/<\/TD><td bgcolor=$color>/g ;
	print "<tr><td bgcolor=\"$color\" $spanString>$string</td></tr>\n";
}

sub printHeadRow {
  	my $string = shift;
 	my $color = shift;
 	my $span = shift;
	my $spanString = "";
	if ( $color eq "" ) { $color="#FFFFFF"; }
	if ( $span ne "" ) { $spanString="colspan=$span"; }
	$string =~ s/\n|\t//g ;
	$string =~ s/,/<\/TH><th bgcolor=$color>/g ;
	print "<tr><th bgcolor=\"$color\" $spanString>$string</th></tr>\n";
}

sub cssPrintRow {
  	my $string = shift;
 	my $style = shift;
 	my $span = shift;
	$string =~ s/\n|\t//g ;
	$string =~ s/,/<\/TD><td class=\"$style\">/g ;
	&printIndent;
	print "<tr><td class=\"$style\" colspan=\"$span\">$string</td></tr>";
	print "\n";
}

sub newLine {
  print "\n";
}

sub printCGIENV {
	my $name;
	foreach $name (sort ( keys (%ENV) ) )  {
		#Only write non blank events 
		tableStart("white");
		printRow("CGI Parameter: $name = $ENV{$name}");
	} # foreach $name
}

sub errorMessage {
	my $string = shift;
	my $color = shift;
	my $colspan = shift;

	&cellStart($color,$colspan);
	&tableStart("white");
	&printHeadRow("$string",$color);
	&tableEnd;
	&cellEnd;
}

sub cssTableStart {
  	my $style = shift;
  	if ( $style ne "" ) { $style = " class=\"$style\""; }
	print "\n";
	&printIndent;
	++$refer::indent;
	print "<table$style>\n";
}

sub cssCellStart {
 	my $style = shift;
 	my $span = shift;
 	my $width = shift;

  	if ( $style ne "" ) { $style = " class=\"$style\""; }
	if ( $span ne "" ) { $span = " colspan=\"$span\""; }
	#else { $span = " colspan=\"1\""; }

  	if ( $width ne "" ) { $width = " width=\"$width%\""; }
	#if ( $width eq "" ) { $width = " width=\"100%\""; }

	&printIndent;
	++$refer::indent;
	print "<td$style$span$width>"; 
}

sub cssPrintCell {
 	my $style = shift;
  	my $string = shift;
 	my $span = shift;
 	my $width = shift;

  	if ( $style ne "" ) { $style = " class=\"$style\""; }
	if ( $span ne "" ) { $span = " colspan=\"$span\""; }
	if ( $width ne "" ) { $width = " width=$width%"; }

	&printIndent;
	print "<td$style$span$width>$string</td>";
}


1;
