#!/usr/bin/perl
#
#    summary.pl - NMIS CGI Program - Network Mangement Information System
#    Copyright (C) 2001 Keith Sinclair <keith@sinclair.org.au>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#*****************************************************************************
use lib "/usr/share/nmis/lib";
# 
#****** Shouldn't be anything else to customise below here *******************

use Time::ParseDate; 
use strict;
use NMIS;
use func;
use csv;
 
# This reads in the information sent when the user pressed Submit
my %FORM = getCGIForm($ENV{'QUERY_STRING'});
my $scriptname = $ENV{SCRIPT_NAME};

# Break the queary up for the names
my $query = $FORM{query};
my $group = $FORM{group};
my $type = $FORM{type};
my $node = $FORM{node};
# Allow program to use other configuration files
my $conf;
if ( $FORM{file} ne "" ) { $conf = $FORM{file}; }
else { $conf = "nmis.conf"; }
my $configfile = "/etc/nmis/$conf";
if ( -f $configfile ) { loadConfiguration($configfile); }
else { die "Can't access configuration file $configfile.\n"; }

my $version = "1.00";

loadNodeDetails;

my $title = "NMIS Map";
&drawMap;

exit 0;

sub drawMap {
	my $configfile = shift;

	my $net_status;
	my $net_color;
	my $group_status;
	my $net_icon;

	my $group = shift;
	my $start_time = shift;
	my $end_time = shift;
	my %summaryHash;
	my $groupStatus;
	my %map_data;
	my $div;
	my $col; 
	my $row;
	
	if ( $start_time eq "" ) { $start_time = "-8 hours"; }
	if ( $end_time eq "" ) { $end_time = time; }
        %map_data = loadCSV($NMIS::config{Map_Table},$NMIS::config{Map_Key});
	$net_status = overallNodeStatus($group);
	$net_color = eventColor($net_status);

	print <<EOHTML;
Content-type: text/html\n
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<title>$title</title>
<META HTTP-EQUIV=Refresh CONTENT=300>
<STYLE TYPE="text/css">
<!--
body {
	background-color : $net_color;
	color : black;
	font-family : Arial, Helv;
	font-size : xx-small;
	font-style : normal;
	font-variant : normal;
	text-decoration : none
}
.big {
	font-size : x-large;
}
EOHTML
	#create DIV styles for map placement later
       	print "\#Map { position: absolute; left:20; top:20; z-index:1; }\n";
	#create DIV styles for Summary table placement later
	print "\#Summary { position: absolute; left:600; top:20; z-index:3; }\n";
	#DIV's for group icons.
        foreach $group (sort (keys %map_data) ) {
        	# DIV for icon
        	print "\#$map_data{$group}{Group} { position: absolute; left:$map_data{$group}{Column}; top:$map_data{$group}{Row}; z-index:2; }\n";
        	# DIV for words
        	$div = $map_data{$group}{Group}."_Text";
        	$col = $map_data{$group}{Column} + 20;
        	$row = $map_data{$group}{Row} + 15;
        	print "\#$div { position: absolute; left:$col; top:$row; z-index:2; }\n";
	}
	
	# Start the next bit of HTML
	print <<EOHTML;
-->
</STYLE>
</HEAD>
<BODY>

EOHTML
	# Draw the background Map Icon
	print "<DIV ID=\"Map\"><a href=\"$NMIS::config{nmis}\"><img border=\"0\" src=\"$NMIS::config{map_icon}\"></a></DIV>\n";

        foreach $group (sort (keys %map_data) ) {
		
		# Get the groups status and assisgn an icon to use.
		$group_status = overallNodeStatus($map_data{$group}{Group});
		if ( $group_status =~ /normal/i ) { $net_icon = $NMIS::config{normal_net_icon}; }
		elsif ( $group_status =~ /fatal/i ) { $net_icon = $NMIS::config{fatal_net_icon}; }
		elsif ( $group_status =~ /critical/i ) { $net_icon = $NMIS::config{critical_net_icon}; }
		elsif ( $group_status =~ /major/i ) { $net_icon = $NMIS::config{major_net_icon}; }
		elsif ( $group_status =~ /minor/i ) { $net_icon = $NMIS::config{minor_net_icon}; }
		elsif ( $group_status =~ /warning/i ) { $net_icon = $NMIS::config{minor_net_icon}; }
		elsif ( $group_status =~ /alert/i ) { $net_icon = $NMIS::config{minor_net_icon}; }
		else { $net_icon = $NMIS::config{unknown_net_icon}; }
	
		#Draw icon		
		print "<DIV ID=\"$map_data{$group}{Group}\">\<A HREF=\"$NMIS::config{nmis}?group=$map_data{$group}{Group}\"><img border=\"0\" src=\"$net_icon\"></A></DIV>\n";
		#Draw words		
        	$div = $map_data{$group}{Group}."_Text";
		print "<DIV ID=\"$div\"><A HREF=\"$NMIS::config{nmis}?group=$map_data{$group}{Group}\">$map_data{$group}{Group}<BR>$group_status</A></DIV>\n";
	}
	#Draw a summary table.
	%summaryHash = &getGroupSummary("",$start_time,$end_time);

	print <<EOHTML;
<DIV ID="Summary">
<table bgcolor=white>
<tr class="big" bgcolor="$net_color"><td>Status</td><td>$net_status</td></tr>
<tr class="big" bgcolor="$summaryHash{average}{metric_color}"><td>metric</td><td>$summaryHash{average}{metric}</td></tr>
<tr class="big" bgcolor="$summaryHash{average}{reachable_color}"><td>reachable</td><td>$summaryHash{average}{reachable}</td></tr>
<tr class="big" bgcolor="$summaryHash{average}{available_color}"><td>available</td><td>$summaryHash{average}{available}</td></tr>
<tr class="big" bgcolor="$summaryHash{average}{health_color}"><td>health</td><td>$summaryHash{average}{health}</td></tr>
<tr class="big" bgcolor="$summaryHash{average}{response_color}"><td>response</td><td>$summaryHash{average}{response} ms</td></tr>
</table>
</DIV>
EOHTML
	# Finish the page off
	print <<EOHTML;
</BODY>
</HTML>
EOHTML
}
