#!/usr/bin/perl
#
#    query.pl - NMIS CGI Program - Network Mangement Information System
#    Copyright (C) 2001 Keith Sinclair <keith@sinclair.org.au>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#*****************************************************************************
use lib "/usr/share/nmis/lib";

my $configfile = "/etc/nmis/nmis.conf";
# 
#****** Shouldn't be anything else to customise below here *******************

use Time::ParseDate; 
use strict;
use NMIS;
use func;
 
# This reads in the information sent when the user pressed Submit
my %FORM = getCGIForm($ENV{'QUERY_STRING'});
my $scriptname = $ENV{SCRIPT_NAME};

# Break the queary up for the names
my $query = $FORM{query};
my $group = $FORM{group};
my $type = $FORM{type};
my $node = $FORM{node};
# Allow program to use other configuration files
if ( $FORM{file} ne "" ) { $configfile = $FORM{file}; }

loadConfiguration($configfile);

my $version = "1.00";

&startTextPage;

# make the default action to be info
if 	( $query eq "status" and $node ne "" ) 	{ &nodestatus($node); }
elsif 	( $query eq "status" ) 			{ &groupstatus($group); }
elsif 	( $query eq "list" ) 			{ &grouplist($group); }
else 						{ &noquery; }

print "end\n";
exit 0;

sub grouplist {
	my $group = shift;
	my $node;
	my $event_status;
	my $event_color;
	my $groupStatus;

	loadNodeDetails;
	loadEventState;

	print "query=list\n";
	print "group=$group\n";
	print "version=$version\n";
	# Insert some nice status info about the devices for the summary menu.
	if ( $group eq "" ) {
	        foreach $group (sort ( keys (%NMIS::groupTable) ) ) {
			$groupStatus = overallNodeStatus($group);
			print "group=$group:$groupStatus\n";
		}
	}
	foreach $node (sort ( keys (%NMIS::nodeTable) ) ) {
		#Only do the group
		if ( $group eq $NMIS::nodeTable{$node}{group} or $group eq "") {
			if ( eventExist($node,"Node Down","") eq "true" ) {
				($event_status,$event_color) = eventLevel("Node Down",$NMIS::nodeTable{$node}{role});
			}
			else {
				($event_status,$event_color) = eventLevel("Node Up",$NMIS::nodeTable{$node}{role});
			}
			print "node=$node:$event_status\n";	
		}
	}
}

sub groupstatus {
	my $group = shift;
	my $start_time = shift;
	my $end_time = shift;
	my %summaryHash;
	my $groupStatus;

	if ( $start_time eq "" ) { $start_time = "-8 hours"; }
	if ( $end_time eq "" ) { $end_time = time; }

	%summaryHash = &getGroupSummary("","-16 hours","-8 hours");
	$groupStatus = overallNodeStatus($group);

	print "query=status\n";
	print "group=$group\n";
	print "status=$groupStatus\n";
	print "reachable=$summaryHash{average}{reachable}\n";
	print "available=$summaryHash{average}{available}\n";
	print "health=$summaryHash{average}{health}\n";
	print "response=$summaryHash{average}{response}\n";
	print "metric=$summaryHash{average}{metric}\n";
	print "count=$summaryHash{average}{count}\n";
}

sub nodestatus {
	my $node = shift;
	my $start_time = shift;
	my $end_time = shift;
	
	my @tmparray;
	my @tmpsplit;
	my %summaryHash;
	my $overallStatus;
	my $reportStats;
	my $index;

	if ( $start_time eq "" ) { $start_time = "-8 hours"; }
	if ( $end_time eq "" ) { $end_time = time; }

	loadEventState;
	if ( eventExist($node,"Node Down","") eq "true" ) {
		($summaryHash{$node}{event_status},$summaryHash{$node}{event_color}) = eventLevel("Node Down",$NMIS::nodeTable{$node}{role});
	}
	else {
		($summaryHash{$node}{event_status},$summaryHash{$node}{event_color}) = eventLevel("Node Up",$NMIS::nodeTable{$node}{role});
	}

	loadSystemFile($node);
	$reportStats = summaryStats($node,"health",$start_time,$end_time);
	if ( $reportStats != 1 and $reportStats ne "" ) {
		@tmparray = split " ","@$reportStats";
		for ( $index = 0; $index <= $#tmparray; ++$index ) {
			@tmpsplit = split "=",@tmparray[$index];
			$summaryHash{$node}{$tmpsplit[0]} = $tmpsplit[1];
		}
	}
	$summaryHash{$node}{metric} = sprintf("%.3f",( ( $summaryHash{$node}{reachable} + $summaryHash{$node}{available} + $summaryHash{$node}{health} ) / 3) );
	#$overallStatus = overallNodeStatus($group);

	print "query=status\n";
	print "node=$node\n";
	print "version=$version\n";
	print "status=$summaryHash{$node}{event_status}\n";
	print "reachable=$summaryHash{$node}{reachable}\n";
	print "available=$summaryHash{$node}{available}\n";
	print "health=$summaryHash{$node}{health}\n";
	print "response=$summaryHash{$node}{response}\n";
	print "metric=$summaryHash{$node}{metric}\n";
}

sub noquery {
	print "ERROR: no query requested\n";	
	print "operations supported:\n";
	print "http://server/$ENV{SCRIPT_NAME}?query=list\n";
	print "http://server/$ENV{SCRIPT_NAME}?query=list&group=<name>\n";
	print "http://server/$ENV{SCRIPT_NAME}?query=status\n";
	print "http://server/$ENV{SCRIPT_NAME}?query=status&group=<name>\n";
	print "http://server/$ENV{SCRIPT_NAME}?query=status&node=<name>\n";
}

sub startTextPage {
	print "Content-type: text/plain\n\n";
}
